#include "pch.h"
#include "AliceVsisRenderer.h"

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::UI::Xaml::Media::Imaging;

AliceVsisRenderer::AliceVsisRenderer(VirtualSurfaceImageSource^ vsis) : 
    VirtualSurfaceImageSourceRenderer(vsis),
    m_width(0),
    m_pixelWidth(0),
    m_pixelHeight(0)
{
}

void AliceVsisRenderer::SetText(std::vector<std::wstring> lines)
{
    m_aliceParagraphGenerator.SetText(lines);
    ResizeAndInvalidate();
}

void AliceVsisRenderer::SetWidth(float width)
{
    this->m_width = width;
    ResizeAndInvalidate();
}

// Move to VirtualSurfaceImageSource etc, but make virtual

void AliceVsisRenderer::SetDpi(float dpi)
{
    VirtualSurfaceImageSourceRenderer::SetDpi(dpi);
    ResizeAndInvalidate();
}

void AliceVsisRenderer::ResizeAndInvalidate()
{
    if (m_width == 0 || m_aliceParagraphGenerator.ParagraphCount() == 0)
        return;

    // Hard code 50-DIU left and right margin
    float height = m_aliceParagraphGenerator.SetWidth(m_width - 100);

    // Calculate pixel width and height
    m_pixelWidth = (int) (m_dpi * m_width / 96);
    m_pixelHeight = (int) (m_dpi * height / 96);

    // Resize the VSIS
    m_vsisNative->Resize(m_pixelWidth, m_pixelHeight);

    // Invalidate the VSIS
    RECT rect;
    rect.left = 0;
    rect.top = 0;
    rect.right = m_pixelWidth;
    rect.bottom = m_pixelHeight;
    m_vsisNative->Invalidate(rect);
}

SIZE AliceVsisRenderer::GetPixelSize()
{
    SIZE pixelSize = { m_pixelWidth, m_pixelHeight };
    return pixelSize;
}

void AliceVsisRenderer::CreateDeviceResources()
{
    VirtualSurfaceImageSourceRenderer::CreateDeviceResources();

    HRESULT hr = m_d2dContext->CreateSolidColorBrush(ColorF(ColorF::Black), &m_blackBrush);

    if (FAILED(hr))
        throw ref new COMException(hr, "d2dContext->CreateSolidColorBrush failure");
}

void AliceVsisRenderer::Render(D2D1_RECT_F updateRect)
{
    m_d2dContext->Clear(ColorF(ColorF::White));

    std::vector<Paragraph> paragraphs = m_aliceParagraphGenerator.GetParagraphs();
    D2D1_POINT_2F origin = Point2F(50, (float)-updateRect.top);
    float outputHeight = (float) (updateRect.bottom - updateRect.top);

    for (Paragraph paragraph : paragraphs)
    {
        if (origin.y + paragraph.TextHeight + paragraph.SpaceAfter > 0)
            m_d2dContext->DrawTextLayout(origin, paragraph.TextLayout.Get(), m_blackBrush.Get());

        origin.y += paragraph.TextHeight + paragraph.SpaceAfter;

        if (origin.y > outputHeight)
            break;
    }
}
