#include "pch.h"
#include "ParagraphFormattedAliceRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace ParagraphFormattedAlice;

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;

ParagraphFormattedAliceRenderer::ParagraphFormattedAliceRenderer(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_deviceResources(deviceResources)
{
    CreateDeviceDependentResources();
}

void ParagraphFormattedAliceRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->
        CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black),
        &m_blackBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->
        CreateDrawingStateBlock(&m_stateBlock)
        );
}

void ParagraphFormattedAliceRenderer::CreateWindowSizeDependentResources()
{
    Windows::Foundation::Size windowBounds = m_deviceResources->GetOutputBounds();
    m_aliceParagraphGenerator.SetWidth(windowBounds.Width - 100);
}

void ParagraphFormattedAliceRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void ParagraphFormattedAliceRenderer::SetAliceText(std::vector<std::wstring> lines)
{
    m_aliceParagraphGenerator.SetText(lines);
    CreateWindowSizeDependentResources();
}

void ParagraphFormattedAliceRenderer::Render()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();

    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(ColorF(ColorF::White));
    context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    std::vector<Paragraph> paragraphs = m_aliceParagraphGenerator.GetParagraphs();
    D2D1_POINT_2F origin = Point2F(50, 0);

    for (Paragraph paragraph : paragraphs)
    {
        context->DrawTextLayout(origin, paragraph.TextLayout.Get(), m_blackBrush.Get());
        origin.y += paragraph.TextHeight + paragraph.SpaceAfter;
    }

    // We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());
}

