﻿#include "pch.h"
#include "ParagraphFormattedAliceMain.h"

using namespace ParagraphFormattedAlice;

// Loads and initializes application assets when the application is loaded.
ParagraphFormattedAliceMain::ParagraphFormattedAliceMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
    // Create renderer
    m_paragraphFormattedAliceRenderer = std::unique_ptr<ParagraphFormattedAliceRenderer>(new ParagraphFormattedAliceRenderer(m_deviceResources));
    m_paragraphFormattedAliceRenderer->CreateWindowSizeDependentResources();

	m_deviceResources->RegisterDeviceNotify(this);
}

ParagraphFormattedAliceMain::~ParagraphFormattedAliceMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void ParagraphFormattedAliceMain::CreateWindowSizeDependentResources() 
{
    m_paragraphFormattedAliceRenderer->CreateWindowSizeDependentResources();
    m_paragraphFormattedAliceRenderer->Render();
    m_deviceResources->Present();
}

// Called from rendering loop in DirectXPage but disabled
void ParagraphFormattedAliceMain::Update() 
{
}

// Called from rendering loop in DirectXPage but disabled
// Returns true if the frame was rendered and is ready to be displayed.
bool ParagraphFormattedAliceMain::Render() 
{
    return false;
}

// Notifies renderers that device resources need to be released.
void ParagraphFormattedAliceMain::OnDeviceLost()
{
    m_paragraphFormattedAliceRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void ParagraphFormattedAliceMain::OnDeviceRecreated()
{
    m_paragraphFormattedAliceRenderer->CreateDeviceDependentResources();
    CreateWindowSizeDependentResources();
}

void ParagraphFormattedAliceMain::SetAliceText(std::vector<std::wstring> lines)
{
    m_paragraphFormattedAliceRenderer->SetAliceText(lines);
    m_paragraphFormattedAliceRenderer->Render();
    m_deviceResources->Present();
}
