#include "pch.h"
#include "PlainTextAliceRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace PlainTextAlice;

using namespace D2D1;
using namespace Platform;

PlainTextAliceRenderer::PlainTextAliceRenderer(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_text(L""),
    m_deviceResources(deviceResources)
{
    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->
            CreateTextFormat(L"Century Schoolbook",
                             nullptr,
                             DWRITE_FONT_WEIGHT_NORMAL,
                             DWRITE_FONT_STYLE_NORMAL,
                             DWRITE_FONT_STRETCH_NORMAL,
                             24.0f,
                             L"en-US",
                             &m_textFormat)
    );

    CreateDeviceDependentResources();
}

void PlainTextAliceRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->
            CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), 
                                  &m_blackBrush)
    );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->
            CreateDrawingStateBlock(&m_stateBlock)
    );
}

void PlainTextAliceRenderer::CreateWindowSizeDependentResources()
{
    Windows::Foundation::Size windowBounds = m_deviceResources->GetOutputBounds();
    m_layoutRect = RectF(50, 0, windowBounds.Width - 50,  windowBounds.Height);
}

void PlainTextAliceRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void PlainTextAliceRenderer::SetAliceText(std::wstring text)
{
    m_text = text;
}

void PlainTextAliceRenderer::Render()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();

    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(ColorF(ColorF::White));
    context->SetTransform(m_deviceResources->GetOrientationTransform2D());
    context->DrawText(m_text.c_str(), 
                      m_text.length(), 
                      m_textFormat.Get(), 
                      m_layoutRect, 
                      m_blackBrush.Get(), 
                      D2D1_DRAW_TEXT_OPTIONS_NONE, 
                      DWRITE_MEASURING_MODE_NATURAL);

    // We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());
}

