﻿#include "pch.h"
#include "PlainTextAliceMain.h"


using namespace PlainTextAlice;

// Loads and initializes application assets when the application is loaded.
PlainTextAliceMain::PlainTextAliceMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
    // Create renderer
    m_PlainTextAliceRenderer = std::unique_ptr<PlainTextAliceRenderer>(new PlainTextAliceRenderer(m_deviceResources));

	m_deviceResources->RegisterDeviceNotify(this);
}

PlainTextAliceMain::~PlainTextAliceMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void PlainTextAliceMain::CreateWindowSizeDependentResources() 
{
    m_PlainTextAliceRenderer->CreateWindowSizeDependentResources();
    m_PlainTextAliceRenderer->Render();
    m_deviceResources->Present();
}

void PlainTextAliceMain::Update() 
{
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool PlainTextAliceMain::Render() 
{
    return false;
}

// Notifies renderers that device resources need to be released.
void PlainTextAliceMain::OnDeviceLost()
{
    m_PlainTextAliceRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void PlainTextAliceMain::OnDeviceRecreated()
{
    m_PlainTextAliceRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void PlainTextAliceMain::SetAliceText(std::wstring text)
{
    m_PlainTextAliceRenderer->SetAliceText(text);
    m_PlainTextAliceRenderer->Render();
    m_deviceResources->Present();
}
