#include "pch.h"
#include "FirstLineIndent.h"

FirstLineIndent::FirstLineIndent(int indent) : m_refCount(0)
{
    m_indent = indent;
}

// IUnknown methods
ULONG STDMETHODCALLTYPE FirstLineIndent::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE FirstLineIndent::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE FirstLineIndent::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(IDWriteInlineObject))
    {
        *ppOutput = static_cast<IDWriteInlineObject*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

HRESULT STDMETHODCALLTYPE FirstLineIndent::GetMetrics(_Out_ DWRITE_INLINE_OBJECT_METRICS * metrics)
{
    metrics->width = (float)m_indent;
    metrics->height = 1;
    metrics->baseline = 1;
    metrics->supportsSideways = true;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE FirstLineIndent::GetOverhangMetrics(_Out_ DWRITE_OVERHANG_METRICS *overhangs)
{
    overhangs->left = 0;
    overhangs->top = 0;
    overhangs->right = 0;
    overhangs->bottom = 0;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE FirstLineIndent::GetBreakConditions(_Out_ DWRITE_BREAK_CONDITION * breakConditionBefore,
                                                              _Out_ DWRITE_BREAK_CONDITION * breakConditionAfter)
{
    *breakConditionBefore = DWRITE_BREAK_CONDITION_NEUTRAL;
    *breakConditionAfter = DWRITE_BREAK_CONDITION_NEUTRAL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE FirstLineIndent::Draw(void * clientDrawingContext,
                                                IDWriteTextRenderer * renderer,
                                                FLOAT originX,
                                                FLOAT originY,
                                                BOOL isSideways,
                                                BOOL isRightToLeft,
                                                IUnknown * clientDrawingEffect)
{
    return S_OK;
}

