#pragma once

#include <string>
#include "..\DeviceResources.h"

#include "Paragraph.h"
#include "AliceParagraphGenerator.h"
#include "FirstLineIndent.h"

namespace ScrollableAlice
{
    class ScrollableAliceRenderer
    {
    public:
        ScrollableAliceRenderer(const std::shared_ptr<DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void ReleaseDeviceDependentResources();
        void Render();

        // Method to set text for rendering
        void SetAliceText(std::vector<std::wstring> lines);
        float GetTotalTextHeight();
        void SetScrollOffset(float scrollOffset);

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DeviceResources> m_deviceResources;

        // Helper class to format lines into Paragraphs        
        AliceParagraphGenerator m_aliceParagraphGenerator;

        // Device-dependent resources 
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blackBrush;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;

        float m_totalTextHeight;
        float m_scrollOffset;
    };
}