﻿#include "pch.h"
#include "DirectXPage.xaml.h"

using namespace ScrollableAlice;

using namespace Concurrency;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::System::Threading;
using namespace Windows::UI::Core;
using namespace Windows::UI::Input;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

DirectXPage::DirectXPage():
	m_windowVisible(true)
{
	InitializeComponent();

	// Register event handlers for page lifecycle.
	CoreWindow^ window = Window::Current->CoreWindow;

	window->VisibilityChanged +=
		ref new TypedEventHandler<CoreWindow^, VisibilityChangedEventArgs^>(this, &DirectXPage::OnVisibilityChanged);

	DisplayInformation^ currentDisplayInformation = DisplayInformation::GetForCurrentView();

	currentDisplayInformation->DpiChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDpiChanged);

	currentDisplayInformation->OrientationChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnOrientationChanged);

	DisplayInformation::DisplayContentsInvalidated +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDisplayContentsInvalidated);

	swapChainPanel->CompositionScaleChanged += 
		ref new TypedEventHandler<SwapChainPanel^, Object^>(this, &DirectXPage::OnCompositionScaleChanged);

	swapChainPanel->SizeChanged +=
		ref new SizeChangedEventHandler(this, &DirectXPage::OnSwapChainPanelSizeChanged);

    // ----------------------------------------------------------------------
    // For ScrollableAlice project, CompositionTarget::Rendering is NOT used!
    // ----------------------------------------------------------------------

	// Register the rendering event, called every time XAML renders the screen.
	// m_eventToken = CompositionTarget::Rendering::add(ref new EventHandler<Object^>(this, &DirectXPage::OnRendering));

	// Whenever this screen is not being used anymore, you can unregister this event with the following line:
	// CompositionTarget::Rendering::remove(m_eventToken);

	// Disable all pointer visual feedback for better performance when touching.
	auto pointerVisualizationSettings = PointerVisualizationSettings::GetForCurrentView();
	pointerVisualizationSettings->IsContactFeedbackEnabled = false; 
	pointerVisualizationSettings->IsBarrelButtonFeedbackEnabled = false;

	// At this point we have access to the device. 
	// We can create the device-dependent resources.
	m_deviceResources = std::make_shared<DeviceResources>();
	m_deviceResources->SetWindow(Window::Current->CoreWindow, swapChainPanel);

	// Register our SwapChainPanel to get independent input pointer events
	auto workItemHandler = ref new WorkItemHandler([this] (IAsyncAction ^)
	{
		// The CoreIndependentInputSource will raise pointer events for the specified device types on whichever thread it's created on.
		CoreIndependentInputSource^ coreInput = swapChainPanel->CreateCoreIndependentInputSource(
			Windows::UI::Core::CoreInputDeviceTypes::Mouse |
			Windows::UI::Core::CoreInputDeviceTypes::Touch |
			Windows::UI::Core::CoreInputDeviceTypes::Pen
			);

		// Register for pointer events, which will be raised on the background thread.
		coreInput->PointerPressed += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerPressed);
		coreInput->PointerMoved += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerMoved);
		coreInput->PointerReleased += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerReleased);

        // -------------------------------------------------------------------------------
        // For ScrollableAlice project, the PointerWheelChanged event is also handled, and
        //      a GestureRecognizer is also created for ManipulationUpdated event
        // -------------------------------------------------------------------------------
        coreInput->PointerWheelChanged += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerWheelChanged);

        m_gestureRecognizer = ref new GestureRecognizer();
        m_gestureRecognizer->GestureSettings = GestureSettings::ManipulationTranslateX | GestureSettings::ManipulationTranslateY | GestureSettings::ManipulationTranslateInertia;
        m_gestureRecognizer->ManipulationUpdated +=
            ref new TypedEventHandler<GestureRecognizer^, ManipulationUpdatedEventArgs^>(this, &DirectXPage::OnManipulationUpdated);

		// Begin processing input messages as they're delivered.
		coreInput->Dispatcher->ProcessEvents(CoreProcessEventsOption::ProcessUntilQuit);
	});

	// Run task on a dedicated high priority background thread.
	m_inputLoopWorker = ThreadPool::RunAsync(workItemHandler, WorkItemPriority::High, WorkItemOptions::TimeSliced);

	m_main = std::unique_ptr<ScrollableAliceMain>(new ScrollableAliceMain(m_deviceResources));

    // ------------------------------------
    // Load in the text for ScrollableAlice
    // ------------------------------------
    Loaded += ref new RoutedEventHandler([this](Object^, Object^)
    {
        create_task(PathIO::ReadLinesAsync("ms-appx:///Text/AliceChapterVII-B.txt",
            UnicodeEncoding::Utf8))
            .then([this](IVector<String^>^ lines)
        {
            std::vector<std::wstring> vLines;

            for (String^ line : lines)
                vLines.push_back(std::wstring(line->Data()));

            critical_section::scoped_lock lock(m_criticalSection);

            m_main->SetAliceText(vLines);
        });
    });
}

// Saves the current state of the app for suspend and terminate events.
void DirectXPage::SaveInternalState(IPropertySet^ state)
{
	m_deviceResources->Trim();

	// Put code to save app state here.
}

// Loads the current state of the app for resume events.
void DirectXPage::LoadInternalState(IPropertySet^ state)
{
	// Put code to load app state here.
}

// Called every time XAML decides to render a frame (disabled for ScrollableAlize)
void DirectXPage::OnRendering(Object^ sender, Object^ args)
{
	if (m_windowVisible)
	{
		m_main->Update();

		if (m_main->Render())
		{
			m_deviceResources->Present();
		}
	}
}

// Window event handlers.
void DirectXPage::OnVisibilityChanged(CoreWindow^ sender, VisibilityChangedEventArgs^ args)
{
	m_windowVisible = args->Visible;
}

// Display properties event handlers.
void DirectXPage::OnDpiChanged(DisplayInformation^ sender, Object^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
	m_deviceResources->SetDpi(sender->LogicalDpi);
}

void DirectXPage::OnOrientationChanged(DisplayInformation^ sender, Object^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
	m_deviceResources->UpdateForWindowSizeChange();
	m_main->CreateWindowSizeDependentResources();
}

void DirectXPage::OnDisplayContentsInvalidated(DisplayInformation^ sender, Object^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
	m_deviceResources->ValidateDevice();
}

void DirectXPage::OnPointerPressed(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerPressed event, which will be raised on a background thread.
    m_gestureRecognizer->ProcessDownEvent(e->CurrentPoint);
}

void DirectXPage::OnPointerMoved(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerMoved event, which will be raised on a background thread.
    m_gestureRecognizer->ProcessMoveEvents(e->GetIntermediatePoints());
}

void DirectXPage::OnPointerReleased(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerReleased event, which will be raised on a background thread.
    m_gestureRecognizer->ProcessUpEvent(e->CurrentPoint);
}

void DirectXPage::OnCompositionScaleChanged(SwapChainPanel^ sender, Object^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
    m_deviceResources->CreateWindowSizeDependentResources();
    m_main->CreateWindowSizeDependentResources();
}

void DirectXPage::OnSwapChainPanelSizeChanged(Object^ sender, SizeChangedEventArgs^ e)
{
    critical_section::scoped_lock lock(m_criticalSection);
    m_deviceResources->UpdateForWindowSizeChange();
	m_main->CreateWindowSizeDependentResources();
}

void DirectXPage::OnPointerWheelChanged(Object^ sender, PointerEventArgs^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
    m_main->ScrollVerticalDelta((float)args->CurrentPoint->Properties->MouseWheelDelta / 4);
}

void DirectXPage::OnManipulationUpdated(GestureRecognizer^ sender, ManipulationUpdatedEventArgs^ args)
{
    critical_section::scoped_lock lock(m_criticalSection);
    m_main->ScrollVerticalDelta(args->Delta.Translation.Y);
}