﻿#include "pch.h"
#include "ScrollableAliceMain.h"

using namespace ScrollableAlice;

// Loads and initializes application assets when the application is loaded.
ScrollableAliceMain::ScrollableAliceMain(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_scrollOffset(0),
    m_maxScrollOffset(0),
	m_deviceResources(deviceResources)
{
	// Create renderer
	m_ScrollableAliceRenderer = std::unique_ptr<ScrollableAliceRenderer>(new ScrollableAliceRenderer(m_deviceResources));
	m_ScrollableAliceRenderer->CreateWindowSizeDependentResources();

	m_deviceResources->RegisterDeviceNotify(this);
}

ScrollableAliceMain::~ScrollableAliceMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void ScrollableAliceMain::CreateWindowSizeDependentResources() 
{
    m_ScrollableAliceRenderer->CreateWindowSizeDependentResources();

    if (m_maxScrollOffset > 0)
    {
        float fraction = m_scrollOffset / m_maxScrollOffset;
        CalculateMaxScrollOffset();
        m_scrollOffset = fraction * m_maxScrollOffset;
    }
    else
    {
        CalculateMaxScrollOffset();
    }
    RenderScrolledOutput();
}

// Updates the application state once per frame.
void ScrollableAliceMain::Update() 
{
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool ScrollableAliceMain::Render() 
{
    return false;
}

// Notifies renderers that device resources need to be released.
void ScrollableAliceMain::OnDeviceLost()
{
    m_ScrollableAliceRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void ScrollableAliceMain::OnDeviceRecreated()
{
	m_ScrollableAliceRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

// Methods called from DirectXPage
void ScrollableAliceMain::SetAliceText(std::vector<std::wstring> lines)
{
    m_ScrollableAliceRenderer->SetAliceText(lines);
    CalculateMaxScrollOffset();
    RenderScrolledOutput();
}

void ScrollableAliceMain::ScrollVerticalDelta(float deltaY)
{
    m_scrollOffset -= deltaY;
    RenderScrolledOutput();
}

void ScrollableAliceMain::CalculateMaxScrollOffset()
{
    float totalTextHeight = m_ScrollableAliceRenderer->GetTotalTextHeight();
    m_maxScrollOffset = totalTextHeight - m_deviceResources->GetOutputBounds().Height;
}

void ScrollableAliceMain::RenderScrolledOutput()
{
    m_scrollOffset = max(0, min(m_maxScrollOffset, m_scrollOffset));
    m_ScrollableAliceRenderer->SetScrollOffset(m_scrollOffset);
    m_ScrollableAliceRenderer->Render();
    m_deviceResources->Present();
}
